package com.yihu.hos.central.rest.common.configuration;

import com.yihu.ehr.elasticsearch.ElasticSearchUtil;
import com.yihu.ehr.util.datetime.DateUtil;
import com.yihu.ehr.util.rest.Envelop;
import com.yihu.hos.central.rest.feign.RedisClient;
import com.yihu.hos.central.rest.services.crawler.CrawlerManager;
import com.yihu.hos.central.rest.services.errorUpload.ErrorUploadService;
import com.yihu.hos.central.rest.services.upload.UploadDetailService;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.quartz.SchedulerFactoryBean;

import java.util.*;

/**
 * Created by Administrator on 2017/12/20.
 */
@Configuration
@EnableScheduling
public class SchedulingConfig {

    private static Logger logger = LoggerFactory.getLogger(SchedulingConfig.class);

    @Autowired
    private CrawlerManager crawlerManager;

    @Autowired
    private UploadDetailService uploadDetailService;

    @Autowired
    private ErrorUploadService errorUploadService;

    @Autowired
    private RedisClient redisClient;
    @Autowired
    private ElasticSearchUtil elasticSearchUtil;

    @Bean
    SchedulerFactoryBean schedulerFactoryBean(){
        SchedulerFactoryBean bean = new SchedulerFactoryBean();
        bean.setAutoStartup(true);
        bean.setSchedulerName("UploadResourceScheduler");
        return bean;
    }

    @Scheduled(cron = "0 0 12 * * ?") // 每天凌晨12点执行1次，获取错误ES数据放入重传队列中
    public void pushUploadFail(){
        errorUploadService.pushData2Queue();
    }

    @Scheduled(cron = "0/1 * * * * ?") // 消费错误重传队列，做重传作业
    public void reImport(){
        crawlerManager.failUpload();
    }


    @Scheduled(cron = "0 10 0 * * ?") // 每天晚上 00点10分执行1次，采集当前省平台的上传情况数据
    public void getUploadDetail() throws Exception {
        logger.info("----------------开始定时记录采集上传情况-------------------");
        uploadDetailService.saveOrUpdate(new Date());
        logger.info("----------------定时记录采集上传情况完成-------------------");
    }


    //省平台上传统计 - 历史数据抽取任务
    @Scheduled(fixedDelay = 6000)
    public void proPlatformHistoryUpdate() throws Exception {
        String date = redisClient.get("start_upload_date");
        if (StringUtils.isEmpty(date)) {
            date = "2018-08-08";
            redisClient.set("start_upload_date",date);
        }
        if("2018-07-01".equals(date)){
            return;
        }
        Envelop envelop = uploadDetailService.uploadDatasetDetail(date);
        if (envelop.isSuccessFlg()){
            Date beginDate = DateUtil.strToDate(date);
            Date addDate = DateUtil.addDate(-1, beginDate);
            String endDate = DateUtil.toString(addDate);
            redisClient.set("start_upload_date",endDate);
            System.out.println("抽取"+ date + "数据集统计数据成功====================");
        }

    }


    //省平台上传统计抽取任务
    @Scheduled(fixedDelay = 60000)
    public void provPlatformDelayUpdate() throws Exception {
        Map<String, String> datasetDetails = crawlerManager.getDatasetDetails();
        if (!datasetDetails.isEmpty()) {
            List<Map<String, Object>> indexs = new ArrayList<>();
            List<Map<String, Object>> updates = new ArrayList<>();
            datasetDetails.forEach((key, val) -> {
                String [] _key = key.split(";");
                String [] _val = val.split(";");
                Map<String, Object> map = new HashMap<>();
                StringBuilder id = new StringBuilder();
                id.append(_key[0])
                        .append("_")
                        .append(_key[1])
                        .append("_")
                        .append(_key[3])
                        .append("_")
                        .append(_key[4]);
                Map<String, Object> source = elasticSearchUtil.findById("upload", "qc_dataset_detail", id.toString());
                map.put("_id", id.toString());
                if (source != null) {
                    map.put("count", Integer.parseInt(source.get("count").toString()) +  Integer.parseInt(_val[0]));
                    map.put("row", Integer.parseInt(source.get("row").toString()) + Integer.parseInt(_val[1]));
                    updates.add(map);
                } else {
                    map.put("org_code", _key[0]);
                    map.put("receive_date", _key[1] + " 00:00:00");
                    map.put("event_type", Integer.parseInt(_key[3]));
                    map.put("dataset", _key[4]);
                    map.put("to_platform", _key[5]);
                    map.put("dataset_name", redisClient.get("std_data_set_" + _key[2] + ":" + _key[4] + ":name"));
                    map.put("count", Integer.parseInt(_val[0]));
                    map.put("row", Integer.parseInt(_val[1]));
                    indexs.add(map);
                }
            });
            elasticSearchUtil.bulkIndex("upload", "qc_dataset_detail", indexs);
            elasticSearchUtil.bulkUpdate("upload", "qc_dataset_detail", updates);
        }
    }


}
