package com.yihu.hos.central.rest.common.util;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.yihu.ehr.elasticsearch.ElasticSearchUtil;
import com.yihu.hos.central.rest.models.crawler.statistics.Upload;
import com.yihu.hos.central.rest.services.qc.EsSimplePackage;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.util.springutil.SpringBeanUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.util.CollectionUtils;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;

/**
 * 导入mysql数据库
 */
public class ImportDataToDataBase {

    private static Logger logger = LoggerFactory.getLogger(ImportDataToDataBase.class);


    private static JdbcTemplate tempJdbcTemplate;
    private static ObjectMapper objectMapper = new ObjectMapper();
    private static final String INDEX = "upload";
    private static final String TYPE = "info";
    private static ElasticSearchUtil elasticSearchUtil;


    /**
     * 批量生成sql语法，导入到数据库中
     *
     * @param table     表名
     * @param indexName 索引名
     * @param arrayNode 源数据
     * @return
     * @throws Exception
     */
    public static boolean batchExportSql(String table, String indexName, ArrayNode arrayNode) {
        //生成sql语法
        List<String> sqlList = null;
        try {
            sqlList = genInsertSql(table, indexName, arrayNode);
            int errorCount = exportSqlToOracle(sqlList);//执行sql到oracle
//            exportSqlToMysql(sqlList);//执行sql到mysql
            return true;
        } catch (Exception e) {
            logger.error("导入数据库异常", e);
        }
        return false;
    }


    /**
     * 生成sql语法
     *
     * @param tableName 表名
     * @param indexName 索引名
     * @param arrayNode 源数据
     * @return
     * @throws Exception
     */
    public static List<String> genInsertSql(String tableName, String indexName, ArrayNode arrayNode) throws Exception {
        List<String> sqlList = new ArrayList<>();
        String insertSql = null;
        for (JsonNode jsonNode : arrayNode) {
            if ("EDS_OUTPAT_MEDICAL_RECORD".equalsIgnoreCase(tableName) || "EDS_EM_OBSERVATION_MED_RECORD".equalsIgnoreCase(tableName)) {
                //锐软采集上来,这数据可能没有
                if (jsonNode.get("OUTPAT_FORM_NO") == null || StringUtils.isBlank(jsonNode.get("OUTPAT_FORM_NO").asText())) {
                    continue;
                }
            } else if ("EXAMMASTER".equalsIgnoreCase(tableName) || "LABMASTER".equalsIgnoreCase(tableName)) {
                //锐软采集上来,这数据可能没有
                if (jsonNode.get("REPORT_FORM_NO") == null || StringUtils.isBlank(jsonNode.get("REPORT_FORM_NO").asText())) {
                    continue;
                }
                if (jsonNode.get("PATIENT_ID") == null || StringUtils.isBlank(jsonNode.get("PATIENT_ID").asText())) {
                    continue;
                }
            } else if ("INPATIENT_PROORDER".equalsIgnoreCase(tableName) || "INPATIENT_LONGORDER".equalsIgnoreCase(tableName)) {
                //锐软采集上来,这数据可能没有
                if (jsonNode.get("ORDER_ID") == null || StringUtils.isBlank(jsonNode.get("ORDER_ID").asText())) {
                    continue;
                }
            } else if ("LAB_SUBITEM".equalsIgnoreCase(tableName)) {
                if (jsonNode.get("REPORT_FORM_NO") == null || StringUtils.isBlank(jsonNode.get("REPORT_FORM_NO").asText())) {
                    continue;
                }
            }

            insertSql = "INSERT /*+ IGNORE_ROW_ON_DUPKEY_INDEX(" + tableName + "," + indexName + ")*/ INTO " + tableName + " ";
            Iterator<Map.Entry<String, JsonNode>> fields = jsonNode.fields();
            StringBuilder namesStr = new StringBuilder();
            StringBuilder valuesStrBuf = new StringBuilder();
            if (!jsonNode.has("LAST_UPDATE_DTIME")) {
                namesStr.append("LAST_UPDATE_DTIME").append(",");
                valuesStrBuf.append("SYSDATE").append(",");
            }


            while (fields.hasNext()) {
                Map.Entry<String, JsonNode> next = fields.next();
                String key = next.getKey();
                JsonNode value = next.getValue();
                namesStr.append(key).append(",");
                if (jsonNode.get(key).isInt() || jsonNode.get(key).isDouble() || jsonNode.get(key).isNumber()) {
                    valuesStrBuf.append(value.asText()).append(",");
                } else {
                    if ("SYSDATE".equals(value.asText().toUpperCase())) {
                        valuesStrBuf.append(value.asText()).append(",");
                    } else if (next.getValue().asText().toUpperCase().contains("TO_DATE")) {
                        valuesStrBuf.append(value.asText()).append(",");
                    } else {
                        valuesStrBuf.append("'").append(next.getValue().asText()).append("',");
                    }
                }

            }

            namesStr = new StringBuilder(namesStr.substring(0, namesStr.length() - 1));
            valuesStrBuf = new StringBuilder(valuesStrBuf.substring(0, valuesStrBuf.length() - 1));
            insertSql += "(" + namesStr + ") VALUES (" + valuesStrBuf + ")";
            //logger.info(insertSql);
            sqlList.add(insertSql);
        }

        return sqlList;

    }


    //导入mysql
    public static void exportSqlToMysql(List<String> sqlList) throws SQLException {
        Connection connection = null;
        try {
            //TODO 修改链接
//            String driver = "oracle.jdbc.driver.OracleDriver"; //Oracle
//            String url = "jdbc:oracle:thin:@10.16.11.9:1521/sjpt_p4";
//            String user = "shangrao";
//            String password = "shangrao_2017";

            String driver = "com.mysql.jdbc.Driver"; //mysql
            String url = "jdbc:mysql://171.35.109.39:3310/srareatempfinal?useUnicode=true&characterEncoding=UTF-8&useSSL=false";
            String user = "chenweishan";
            String password = "GP8Qz4qU";
            Class.forName(driver);

            connection = DriverManager.getConnection(url, user, password);

            int index = 0;
            Statement preparedStatement = connection.createStatement();
            for (String sql : sqlList) {
                preparedStatement.addBatch(sql);
                //执行批量执行
                index++;
                if (index % 1000 == 0) {
                    System.out.println("index:" + index);
                    preparedStatement.executeBatch();
                }
            }
            preparedStatement.executeBatch();
            preparedStatement.close();
        } catch (SQLException e) {
            logger.error("写入数据库异常1", e);
        } catch (ClassNotFoundException e) {
            logger.error("写入数据库异常2", e);
        } finally {
            if (connection != null) {
                connection.close();
            }
        }
    }


    //导入oracle
    public static int exportSqlToOracle(List<String> sqlList) {
        String errorSql = "";
        int errorCount = 0;
        for (String sql : sqlList) {

            long t1 = System.currentTimeMillis();
            errorSql = sql;
            try {
                if (tempJdbcTemplate == null) {
                    tempJdbcTemplate = SpringBeanUtil.getService("tempJdbcTemplate");
                }
                tempJdbcTemplate.execute(sql);
            } catch (DuplicateKeyException e) {//数据重复传了
                errorCount++;
                continue;
            } catch (DataIntegrityViolationException e) {//值不能为空等问题,需要排查 数据转换错误,或者数据源本身如此等
                errorCount++;
                recordFailedInsert(null,null,null,e, errorSql);
                continue;
            } catch (Exception e) {
                errorCount++;
                recordFailedInsert(null,null,null,e, errorSql);
                continue;
            }
            long t2 = System.currentTimeMillis();
            System.out.println("导入一条sql耗时:" + (t2 - t1) + "ms");
        }
        return errorCount;
    }

    //写入到省平台时,错误记录到es
    public static void recordFailedInsert(EsSimplePackage pack,String rowKey,String errorMsg,Exception e, String sql) {
        Upload upload = new Upload();
        String id = UUID.randomUUID().toString().replaceAll("-", "");
        upload.set_id(id);
        upload.setOrigin(5);
        upload.setCreate_date(new Date());
        upload.setFailed_count(1);
        upload.setFailed_message(errorMsg+ "--"+e.getMessage());
        upload.setRow_key(rowKey);

        upload.setEvent_date(pack.getEvent_date());
        upload.setPatient_id(pack.getPatient_id());
        upload.setEvent_no(pack.getEvent_no());
        upload.setEvent_type(pack.getEvent_type());
        upload.setPatient_name(pack.getPatient_name());
        upload.setIdcard_no(pack.getIdcard_no());
        upload.setOrg_code(pack.getOrg_code());
        upload.setOrg_name(pack.getOrg_name());

        try {
            if (elasticSearchUtil == null) {
                elasticSearchUtil = SpringBeanUtil.getService("elasticSearchUtil");
            }
            if (e instanceof DataIntegrityViolationException) {
                upload.setFailed_count(3);
            }
            elasticSearchUtil.index(INDEX, TYPE, objectMapper.readValue(objectMapper.writeValueAsString(upload), Map.class));
        } catch (Exception e1) {
            logger.error("插入数据库失败后,保存到es出错", e1);
            logger.error("当前错误sql:" + sql);
        }
    }


    /* *******************************  生成单条sql语法  **********************************************/

    /**
     * 生成单条sql语法，导入到数据库中
     *
     * @param table     表名
     * @param indexName 索引名
     * @param objNode   源数据
     * @return
     * @throws Exception
     */
    public static String exportSql(EsSimplePackage pack, String rowKey, String table, String indexName, JsonNode objNode) {
        String errorMsg = null;
        String sql = null;
        try {
            //生成sql语法
            sql = genInsertSql(table, indexName, objNode);

            //执行sql到oracle
            if (tempJdbcTemplate == null) {
                tempJdbcTemplate = SpringBeanUtil.getService("tempJdbcTemplate");
            }
            tempJdbcTemplate.execute(sql);
        } catch (DuplicateKeyException e) {//数据重复传了
            //TODO  做更新操作
            try {
                logger.info("正在做更新操作");
                sql = genUpdateSql(table,objNode);
                logger.info(sql);
                tempJdbcTemplate.execute(sql);
                logger.info("更新完成");
            } catch (Exception e1) {
                errorMsg = "执行省平台数据更新操作异常:"+e.getMessage();
                recordFailedInsert(pack,rowKey,"执行省平台数据插入异常0",e, sql);
                logger.error("执行省平台数据更新操作异常0:", e);
            }
        } catch (DataIntegrityViolationException e) {//值不能为空等问题,需要排查 数据转换错误,或者数据源本身如此等
            errorMsg = e.getMessage() + "： 值不能为空，数据转换等问题！";
            recordFailedInsert(pack,rowKey,"执行省平台数据插入异常1",e, sql);
            logger.error("执行省平台数据插入异常1:", e);
        } catch (Exception e) {
            errorMsg = e.getMessage() + "： 导入省平台数据库异常！";
            recordFailedInsert(pack,rowKey,"执行省平台数据插入异常2",e, sql);
            logger.error("执行省平台数据插入异常2:", e);
        }
        return errorMsg;
    }

    /**
     * 生成单条sql语法
     *
     * @param tableName 表名
     * @param indexName 索引名
     * @param jsonNode  源数据
     * @return
     * @throws Exception
     */
    public static String genInsertSql(String tableName, String indexName, JsonNode jsonNode) throws Exception {
        String insertSql = "INSERT /*+ IGNORE_ROW_ON_DUPKEY_INDEX(" + tableName + "," + indexName + ")*/ INTO " + tableName + " ";
        Iterator<Map.Entry<String, JsonNode>> fields = jsonNode.fields();
        StringBuilder namesStr = new StringBuilder();
        StringBuilder valuesStrBuf = new StringBuilder();
        if (!jsonNode.has("LAST_UPDATE_DTIME")) {
            namesStr.append("LAST_UPDATE_DTIME").append(",");
            valuesStrBuf.append("SYSDATE").append(",");
        }
        while (fields.hasNext()) {
            Map.Entry<String, JsonNode> next = fields.next();
            String key = next.getKey();
            if ("rowkey".equals(key)) {
                continue;
            }
            JsonNode value = next.getValue();
            namesStr.append(key).append(",");
            if (jsonNode.get(key).isInt() || jsonNode.get(key).isDouble() || jsonNode.get(key).isNumber()) {
                valuesStrBuf.append(value.asText()).append(",");
            } else {
                if ("SYSDATE".equals(value.asText().toUpperCase())) {
                    valuesStrBuf.append(value.asText()).append(",");
                } else if (next.getValue().asText().toUpperCase().contains("TO_DATE")) {
                    valuesStrBuf.append(value.asText()).append(",");
                } else {
                    valuesStrBuf.append("'").append(next.getValue().asText().replaceAll("'", "''")).append("',"); //去除特殊符号‘
                }
            }

        }
        namesStr = new StringBuilder(namesStr.substring(0, namesStr.length() - 1));
        valuesStrBuf = new StringBuilder(valuesStrBuf.substring(0, valuesStrBuf.length() - 1));
        insertSql += "(" + namesStr + ") VALUES (" + valuesStrBuf + ")";
        return insertSql;

    }

    public static List<String> getPrimaryKey(String tableName){
        assert tableName != null && tableName.length()>0;
        String sql = "select cu.COLUMN_NAME from user_cons_columns cu, user_constraints au where cu.constraint_name = au.constraint_name and au.constraint_type = 'P' and au.table_name ='"+tableName.toUpperCase()+"'";
        if(tempJdbcTemplate == null){
            tempJdbcTemplate = SpringBeanUtil.getService("tempJdbcTemplate");
        }
        List<String> maps = tempJdbcTemplate.queryForList(sql,String.class);
        return maps;
    }

    /**
     * 生成单条sql语法
     *
     * @param tableName 表名
     * @param jsonNode  源数据
     * @return
     * @throws Exception
     */
    public static String genUpdateSql(String tableName, JsonNode jsonNode) throws Exception {
        //首先获取主键,根据主键去做更新操作
        List<String> primaryKeys = getPrimaryKey(tableName);
        if(CollectionUtils.isEmpty(primaryKeys)){
            throw new Exception(tableName+"表没有主键,不能通过主键更新数据");
        }
        String sql = "update "+tableName +" set ";
        Iterator<Map.Entry<String, JsonNode>> fields = jsonNode.fields();
        StringBuilder whereSqlStrBuf = new StringBuilder(" where 1=1 ");
        StringBuilder setSqlStrBuf = new StringBuilder();
        while (fields.hasNext()) {
            Map.Entry<String, JsonNode> next = fields.next();
            String key = next.getKey().toUpperCase();
            JsonNode value = next.getValue();
            if(primaryKeys.contains(key)){ //拼接条件
                whereSqlStrBuf.append(" and ").append(key).append("=");
                StringBuilder appendSql = getAppendSql(jsonNode, key, value, next);
                whereSqlStrBuf.append(appendSql);
            } else {//拼接 set 新值
                setSqlStrBuf.append(",").append(key).append("=");
                StringBuilder appendSql = getAppendSql(jsonNode, key, value, next);
                setSqlStrBuf.append(appendSql);
            }
        }
        String completeSql = sql +  setSqlStrBuf.toString().substring(1) + whereSqlStrBuf.toString();
        return completeSql;
    }

    private static StringBuilder getAppendSql(JsonNode jsonNode,String key,JsonNode value,Map.Entry<String, JsonNode> next){
        StringBuilder appendSql = new StringBuilder();
        if (jsonNode.get(key).isInt() || jsonNode.get(key).isDouble() || jsonNode.get(key).isNumber()) {
            appendSql.append(value.asText());
        } else {
            if ("SYSDATE".equals(value.asText().toUpperCase())) {
                appendSql.append(value.asText());
            } else if (next.getValue().asText().toUpperCase().contains("TO_DATE")) {
                appendSql.append(value.asText());
            } else {
                appendSql.append("'").append(next.getValue().asText().replaceAll("'", "''")).append("'"); //去除特殊符号‘
            }
        }
        return appendSql;
    }


    //单条sql导入oracle
    public static String exportSqlToOracle(String errorSql) {
        String errorMsg = null;
        try {
            if (tempJdbcTemplate == null) {
                tempJdbcTemplate = SpringBeanUtil.getService("tempJdbcTemplate");
            }
            tempJdbcTemplate.execute(errorSql);
        } catch (DuplicateKeyException e) {//数据重复传了
            errorMsg = "DuplicateKeyException,数据已上传过省平台！";
        } catch (DataIntegrityViolationException e) {//值不能为空等问题,需要排查 数据转换错误,或者数据源本身如此等
            errorMsg = e.getMessage() + "： 值不能为空，数据转换等问题！";
            recordFailedInsert(null,null,null,e, errorSql);
            logger.error("执行省平台数据插入异常1:", e);
        } catch (Exception e) {
            errorMsg = e.getMessage() + "： 导入省平台数据库异常！";
            recordFailedInsert(null,null,null,e, errorSql);
            logger.error("执行省平台数据插入异常2:", e);
        }

        return errorMsg;
    }

    public static void main(String[] args) {
        String insertSql = "测试'jj_'xxxxxx'";

        System.out.println(insertSql.replaceAll("'", "''"));
    }


}
