package com.yihu.hos.central.rest.controllers;


import com.yihu.hos.central.rest.models.standard.adapter.AdapterDatasetModel;
import com.yihu.hos.central.rest.services.errorUpload.ErrorUploadService;
import com.yihu.hos.central.rest.services.standard.adapter.AdapterDatasetService;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.model.standard.AdapterVersion;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.util.List;

@RequestMapping("/uploadDate")
@RestController("ChangeUploadDateController")
@Api(protocols = "http", value = "清除脏数据", description = "清除脏数据")
public class ChangeUploadDateController {

    private static Logger logger = LoggerFactory.getLogger(ChangeUploadDateController.class);

    @Autowired
    private AdapterDatasetService adapterDatasetService;

    @Autowired
    private JdbcTemplate tempJdbcTemplate;

    @Value("${schemaVersion}")
    private String schemaVersion;

    @Autowired
    private ErrorUploadService errorUploadService;

    @RequestMapping(value = "/clearByOrgCode", method = RequestMethod.DELETE)
    @ApiOperation(value = "根据orgCode清除脏数据", notes = "根据orgCode清除脏数据")
    public boolean clearByOrgCode(@ApiParam(name = "orgCode", value = "orgCode", required = true)
                                  @RequestParam(value = "orgCode") String orgCode) throws IOException {
        logger.info("开始清除脏数据");
        //todo 根据orgCode清除省平台脏数据
        List<AdapterDatasetModel> adapterDatasetModels = adapterDatasetService.getNeedCrawer(new AdapterVersion(schemaVersion));
        for (AdapterDatasetModel adapterDatasetModel : adapterDatasetModels) {
            String table = adapterDatasetModel.getStdDatasetCode();
            try {
                tempJdbcTemplate.execute("DELETE FROM " + table + " WHERE org_code = '" + orgCode + "'");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        logger.info("根据org_code清除脏数据成功");
        return true;
    }

    @RequestMapping(value = "/changeOrgCode", method = RequestMethod.POST)
    @ApiOperation(value = "更改医高专orgCode为省平台机构编码", notes = "更改医高专orgCode为省平台机构编码")
    public boolean changeOrgCode(@ApiParam(name = "oldOrgCode", value = "oldOrgCode", required = true)
                                 @RequestParam(value = "oldOrgCode") String oldOrgCode,
                                 @ApiParam(name = "newOrgCode", value = "newOrgCode", required = true)
                                 @RequestParam(value = "newOrgCode") String newOrgCode) throws IOException {
        logger.info("开始批量更改org_code数据");
        List<AdapterDatasetModel> adapterDatasetModels = adapterDatasetService.getNeedCrawer(new AdapterVersion(schemaVersion));
        for (AdapterDatasetModel adapterDatasetModel : adapterDatasetModels) {
            String table = adapterDatasetModel.getStdDatasetCode();
            try {
                tempJdbcTemplate.execute("UPDATE " + table + " SET org_code='" + newOrgCode + "' WHERE org_code = '" + oldOrgCode + "'");
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        logger.info("开始批量更改org_code数据完成");
        return true;
    }


    @RequestMapping(value = "/deleteUploadFail", method = RequestMethod.DELETE)
    @ApiOperation(value = "通过时间字段-删除es中upload索引下type的数据；【请慎用】", notes = "删除es数据；")
    public String deleteUploadFail(
            @ApiParam(name = "type", value = "索引下type名", defaultValue = "upload_fail", required = true)
            @RequestParam(value = "type") String type,
            @ApiParam(name = "dateField", value = "type下时间字段", defaultValue = "upload_date", required = true)
            @RequestParam(value = "dateField") String dateField,
            @ApiParam(name = "startDate", value = "起始时间，格式eg：2018-01-01 00:00:00", defaultValue = "2018-01-01 00:00:00", required = true)
            @RequestParam(value = "startDate") String startDate,
            @ApiParam(name = "endDate", value = "结束时间，格式eg：2018-01-01 00:00:00", required = false)
            @RequestParam(value = "endDate", required = false) String endDate) throws IOException {
       String errorMsg = "";
        try {
            errorUploadService.deleteByUpdateDate(type,dateField,startDate, endDate);
            return "操作成功";
        } catch (Exception e) {
            errorMsg = e.getMessage();
            e.printStackTrace();
        }
        return "删除异常：" + errorMsg;
    }

    @RequestMapping(value = "/deleteEsData", method = RequestMethod.DELETE)
    @ApiOperation(value = "通过条件-删除es数据；【请慎用】", notes = "删除es数据；")
    public String deleteEsData(
            @ApiParam(name = "index", value = "index索引", defaultValue = "upload", required = true)
            @RequestParam(value = "index") String index,
            @ApiParam(name = "type", value = "index索引下type名", defaultValue = "upload_fail", required = true)
            @RequestParam(value = "type") String type,
            @ApiParam(name = "filters", value = "dataset=HDSA00_01;org_code<>xxxx;", required = false)
            @RequestParam(value = "filters",required = false) String filters) throws IOException {
        String errorMsg = "";
        try {
            errorUploadService.deleteEsData(index,type,filters);
            return "操作成功";
        } catch (Exception e) {
            errorMsg = e.getMessage();
            e.printStackTrace();
        }
        return "删除异常：" + errorMsg;
    }

}
