package com.yihu.hos.central.rest.job;

import org.quartz.*;
import org.quartz.impl.matchers.GroupMatcher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.client.ServiceInstance;
import org.springframework.cloud.client.discovery.DiscoveryClient;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import static org.quartz.JobBuilder.newJob;
import static org.quartz.TriggerBuilder.newTrigger;

/**
 * Task - 定时检查集群状态，提高解析任务容错率
 * Created by progr1mmer on 2017/12/15.
 */
@Component
public class HealthCheckTask {

    private static String SVR_RESOURCE = "svr-resource";

    private int jobSetSize;
    private int reUploadJobSetSize;
    @Value("${crawler.job.init-size}")
    private int jobInitSize;
    @Value("${crawler.job.max-size}")
    private int jobMaxSize;
    @Value("${crawler.job.cron-exp}")
    private String jobCronExp;
    @Autowired
    private Scheduler scheduler;
    @Autowired
    private DiscoveryClient discoveryClient;

    @PostConstruct
    private void init() {
        //省平台上传任务
        try {
            for (int i = 0; i < jobInitSize; i++) {
                String suffix = UUID.randomUUID().toString().substring(0, 8);
                JobDetail jobDetail = newJob(UploadResourceJob.class)
                        .withIdentity("UploadResourceJob-" + suffix, "UploadResource")
                        .build();
                CronTrigger trigger = newTrigger()
                        .withIdentity("UploadResourceTrigger-" + suffix, "UploadResource")
                        .withSchedule(CronScheduleBuilder.cronSchedule(jobCronExp))
                        .startNow()
                        .build();
                scheduler.scheduleJob(jobDetail, trigger);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        this.jobSetSize = jobInitSize;


        //省平台补传任务
        try {
            for (int i = 0; i < jobInitSize; i++) {
                String suffix = UUID.randomUUID().toString().substring(0, 8);
                JobDetail jobDetail = newJob(SupplementUploadJob.class)
                        .withIdentity("SupplementUploadJob-" + suffix, "SupplementUpload")
                        .build();
                CronTrigger trigger = newTrigger()
                        .withIdentity("SupplementUploadTrigger-" + suffix, "SupplementUpload")
                        .withSchedule(CronScheduleBuilder.cronSchedule(jobCronExp))
                        .startNow()
                        .build();
                scheduler.scheduleJob(jobDetail, trigger);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        this.reUploadJobSetSize = jobInitSize;
    }

    @Scheduled(cron = "0 0/20 * * * ?")
    private void startTask() {
        GroupMatcher groupMatcher = GroupMatcher.groupEquals("UploadResource");
        GroupMatcher reUploadgroupMatcher = GroupMatcher.groupEquals("SupplementUpload");
        //检查微服务信息
        List<ServiceInstance> mgr = discoveryClient.getInstances(SVR_RESOURCE);
        if (mgr.isEmpty()) {
            try {
                Set<JobKey> jobKeySet = scheduler.getJobKeys(groupMatcher);
                if (jobKeySet != null) {
                    for (JobKey jobKey : jobKeySet) {
                        scheduler.deleteJob(jobKey);
                    }
                }
            } catch (SchedulerException e) {
                e.printStackTrace();
            }
            return;
        }

        //省平台上传任务
        scheduleUploadJob(groupMatcher);
       //省平台补传任务
        scheduleReUploadJob(reUploadgroupMatcher);



    }


    /**
     * 省平台上传任务执行
     * @param groupMatcher
     */
    public void scheduleUploadJob(GroupMatcher groupMatcher){
        //省平台上传任务
        try {
            Set<JobKey> jobKeySet = scheduler.getJobKeys(groupMatcher);
            if (jobKeySet != null) {
                int activeCount = jobKeySet.size();
                for (int i = 0; i < jobSetSize - activeCount; i++) {
                    String suffix = UUID.randomUUID().toString().substring(0, 8);
                    JobDetail jobDetail = newJob(UploadResourceJob.class)
                            .withIdentity("UploadResourceJob-" + suffix, "UploadResource")
                            .build();
                    CronTrigger trigger = newTrigger()
                            .withIdentity("UploadResourceTrigger-" + suffix, "UploadResource")
                            .withSchedule(CronScheduleBuilder.cronSchedule(jobCronExp))
                            .startNow()
                            .build();
                    scheduler.scheduleJob(jobDetail, trigger);
                }
            } else {
                for (int i = 0; i < jobSetSize; i++) {
                    String suffix = UUID.randomUUID().toString().substring(0, 8);
                    JobDetail jobDetail = newJob(UploadResourceJob.class)
                            .withIdentity("UploadResourceJob-" + suffix, "UploadResource")
                            .build();
                    CronTrigger trigger = newTrigger()
                            .withIdentity("UploadResourceTrigger-" + suffix, "UploadResource")
                            .withSchedule(CronScheduleBuilder.cronSchedule(jobCronExp))
                            .startNow()
                            .build();
                    scheduler.scheduleJob(jobDetail, trigger);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     *  省平台补传任务执行
     * @param groupMatcher
     */
    public void scheduleReUploadJob(GroupMatcher groupMatcher){
        //省平台上传任务
        try {
            Set<JobKey> jobKeySet = scheduler.getJobKeys(groupMatcher);
            if (jobKeySet != null) {
                int activeCount = jobKeySet.size();
                for (int i = 0; i < jobSetSize - activeCount; i++) {
                    String suffix = UUID.randomUUID().toString().substring(0, 8);
                    JobDetail jobDetail = newJob(UploadResourceJob.class)
                            .withIdentity("SupplementUploadJob-" + suffix, "SupplementUpload")
                            .build();
                    CronTrigger trigger = newTrigger()
                            .withIdentity("SupplementUploadTrigger-" + suffix, "SupplementUpload")
                            .withSchedule(CronScheduleBuilder.cronSchedule(jobCronExp))
                            .startNow()
                            .build();
                    scheduler.scheduleJob(jobDetail, trigger);
                }
            } else {
                for (int i = 0; i < jobSetSize; i++) {
                    String suffix = UUID.randomUUID().toString().substring(0, 8);
                    JobDetail jobDetail = newJob(UploadResourceJob.class)
                            .withIdentity("SupplementUploadJob-" + suffix, "SupplementUpload")
                            .build();
                    CronTrigger trigger = newTrigger()
                            .withIdentity("SupplementUploadTrigger-" + suffix, "SupplementUpload")
                            .withSchedule(CronScheduleBuilder.cronSchedule(jobCronExp))
                            .startNow()
                            .build();
                    scheduler.scheduleJob(jobDetail, trigger);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    public void addJobSize(int addSize) {
        this.jobSetSize += addSize;
        if (this.jobSetSize > jobMaxSize) {
            jobSetSize = jobMaxSize;
        }

        if (this.reUploadJobSetSize > jobMaxSize) {
            reUploadJobSetSize = jobMaxSize;
        }
    }

    public void minusJobSize(int minusSize) {
        this.jobSetSize -= minusSize;
        if (this.jobSetSize < 0) {
            jobSetSize = 0;
        }

        //重传任务数和正常上传任务数一致
        if (this.reUploadJobSetSize < 0) {
            reUploadJobSetSize = 0;
        }
    }

}
