package com.yihu.hos.central.rest.job;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.hos.central.rest.constants.RedisCollection;
import com.yihu.hos.central.rest.feign.SvrResourceServiceClient;
import com.yihu.hos.central.rest.services.crawler.CrawlerManager;
import com.yihu.hos.central.rest.services.qc.EsSimplePackage;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.util.springutil.SpringBeanUtil;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.InterruptableJob;
import org.quartz.JobExecutionContext;
import org.quartz.UnableToInterruptJobException;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.io.Serializable;

/**
 * 上传省平台任务。
 *
 * @author hzy
 * @version 1.0
 * @created 2018.06.12
 */
@Component
@DisallowConcurrentExecution
public class UploadResourceJob implements InterruptableJob {
    private static final Logger logger = LoggerFactory.getLogger(UploadResourceJob.class);
    private static final long DAY = 1000 * 60 * 60 * 24;
    static CrawlerManager crawlerManager = null;

    static {
        crawlerManager = SpringBeanUtil.getService(CrawlerManager.class);
        if (!crawlerManager.getDataForPrepare(null)) {
            logger.info("缓存默认适配版本失败");
            //将机构字典缓存起来
        } else {
            logger.info("缓存默认适配版本成功");
        }
    }

    @Override
    public void interrupt() throws UnableToInterruptJobException {
        logger.info("interrup============================================================");
    }

    @Override
    public void execute(JobExecutionContext context) {

        //该对象要采用名称的方式获取，否则：expected single matching bean but found 3: redisTemplate,sessionRedisTemplate,stringRedisTemplate
        CrawlerManager crawlerManager = SpringBeanUtil.getService(CrawlerManager.class);

        if (!crawlerManager.getDataForPrepare(null)) {
            logger.info("缓存默认适配版本失败");
            throw new RuntimeException("缓存默认适配版本失败");
            //将机构字典缓存起来
        }
        RedisTemplate<String, Serializable> redisTemplate = SpringBeanUtil.getService("redisTemplate");
        ObjectMapper objectMapper = SpringBeanUtil.getService(ObjectMapper.class);
        Serializable serializable = redisTemplate.opsForList().rightPop(RedisCollection.PROVINCIAL_PLATFORM_QUEUE);
        EsSimplePackage pack = null;
        try {
            if (serializable != null) {
                String packStr = serializable.toString();
                pack = objectMapper.readValue(packStr, EsSimplePackage.class);
            }
            if (pack != null) {
                // 上传数据到省平台
                Boolean aBoolean = crawlerManager.collectProcessByRowkey(pack);
                if (!aBoolean) {
                    logger.error("获取资源数据上传省平台失败~！");
                }
            }
        } catch (Exception e) {
           e.printStackTrace();
        }
    }


}
