package com.yihu.hos.central.rest.models.crawler.adapter;

import com.yihu.hos.central.rest.common.format.AdapterBase;
import com.yihu.hos.central.rest.models.standard.adapter.AdapterDatasetModel;
import com.yihu.hos.central.rest.models.standard.adapter.AdapterDictEntryModel;
import com.yihu.hos.central.rest.models.standard.adapter.AdapterMetadataModel;
import com.yihu.hos.central.rest.models.standard.standard.StdDataSetModel;
import com.yihu.hos.central.rest.models.standard.standard.StdMetaDataModel;
import com.yihu.hos.core.datatype.CollectionUtil;
import com.yihu.hos.web.framework.model.standard.AdapterVersion;
import com.yihu.hos.web.framework.model.standard.StandardVersion;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @author Air
 * @version 1.0
 * @created 2015.08.11 11:29
 */
public class AdapterDataSet extends AdapterBase {

    private List<AdapterMetaData> adapterMetaDataList;
    private AdapterDatasetModel adapterDatasetModel;
    private Map<String, AdapterMetaData> adapterMetaDataMap;  //key 为 目标数据元编码
    private Map<String, StdMetaDataModel> targetMetaDataMap; //key 为 目标数据元编码
    private Map<String, StdMetaDataModel> originMetaDataMap; //key 为 源数据元编码
    private String eventNoCode;
    private AdapterVersion adapterVersion;
    private StandardVersion targetVersion;
    private StandardVersion originVersion;
    private StdDataSetModel originDataSetModel; //源数据集
    private StdDataSetModel targetDataSetModel; //目标数据集

    public AdapterDataSet(AdapterDatasetModel adapterDatasetModel, AdapterVersion adapterVersion, StandardVersion originVersion, StandardVersion targetVersion, Map<Long, List<AdapterDictEntryModel>> entryMap) {
        this.adapterDatasetModel = adapterDatasetModel;
        this.adapterVersion = adapterVersion;
        this.targetVersion = targetVersion;
        this.originVersion = originVersion;
        prepareData(entryMap);
    }

    public AdapterDatasetModel getAdapterDataSetT() {
        return adapterDatasetModel;
    }

    public void prepareData(Map<Long, List<AdapterDictEntryModel>> entryMap) {
        try {
            adapterMetaDataList = new ArrayList<>();
            adapterMetaDataMap = new HashMap<>();
            targetMetaDataMap = new HashMap<>();
            originMetaDataMap = new HashMap<>();
            //获取源数据集
            originDataSetModel = getStdDatasetService().get(originVersion,adapterDatasetModel.getAdapterDatasetId());
           //获取目标数据集
//            targetDataSetModel = getStdDatasetService().get(targetVersion,adapterDatasetModel.getStdDatasetId());

            List<AdapterMetadataModel> adapterMetaDataModelList = getAdapterMetadataService().getAdapterMetadataNotNullList(adapterVersion, adapterDatasetModel.getStdDatasetId());
            List<Long> stdMetaDataIdList = new ArrayList<>();
            if (!CollectionUtil.isEmpty(adapterMetaDataModelList)) {
                for (AdapterMetadataModel adapterMetadataModel : adapterMetaDataModelList) {
                    stdMetaDataIdList.add(adapterMetadataModel.getStdMetadataId());
                    AdapterMetaData adapterMetaData = new AdapterMetaData(adapterMetadataModel, adapterVersion, targetVersion,entryMap);
                    adapterMetaDataList.add(adapterMetaData);
                    adapterMetaDataMap.put(adapterMetadataModel.getStdMetadataCode().toUpperCase(), adapterMetaData);

                    StdMetaDataModel originMetaDataModel = getStdMetadataService().get(originVersion.getVersion(), adapterMetadataModel.getAdapterMetadataId());
                    if (originMetaDataModel!=null){
                        originMetaDataMap.put(adapterMetadataModel.getAdapterMetadataCode().toUpperCase(),originMetaDataModel);
                    }

                    StdMetaDataModel targetMetaDataModel = getStdMetadataService().get(targetVersion.getVersion(), adapterMetadataModel.getStdMetadataId());
                    targetMetaDataMap.put(adapterMetadataModel.getStdMetadataCode().toUpperCase(), targetMetaDataModel);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public Map<String, AdapterMetaData> getAdapterMetaDataMap() {
        return adapterMetaDataMap;
    }

    public List<AdapterMetaData> getAdapterMetaDataList() {
        return adapterMetaDataList;
    }

    public StdDataSetModel getOriginDataSetModel() {
        return originDataSetModel;
    }

    public Map<String, StdMetaDataModel> getTargetMetaDataMap() {
        return targetMetaDataMap;
    }

    public Map<String, StdMetaDataModel> getOriginMetaDataMap() {
        return originMetaDataMap;
    }

    public StandardVersion getTargetVersion() {
        return targetVersion;
    }

    public StandardVersion getOriginVersion() {
        return originVersion;
    }
}
