package com.yihu.hos.central.rest.runner;

import com.yihu.hos.central.rest.cache.OrgAdapterCache;
import com.yihu.hos.central.rest.models.standard.adapter.AdapterDictEntryModel;
import com.yihu.hos.central.rest.services.standard.adapter.AdapterDictEntryService;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.model.standard.AdapterVersion;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.Ordered;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class InitAdapter implements ApplicationRunner, Ordered {

    private static final Logger logger = LoggerFactory.getLogger(InitAdapter.class);

    @Value("${schemaVersion}")
    private String schemaVersion;
    @Value("${orgCodeId:5}")
    private Long orgCodeId;
    @Autowired
    private AdapterDictEntryService adapterDictEntryService;

    @Override
    public int getOrder() {
        return 1;
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        //获取适配后的组织机构代码设置成缓存
        Map<String, String> provinceOrgCache = new HashMap<>();//key值存放 标准的org_code   value存放 省平台的org_code
        Map<String,String> stdOrgCodeCache = new HashMap<>(); //key值存放省平台的org_code   value存放 标准的org_code
        List<AdapterDictEntryModel> list = adapterDictEntryService.searchPage(AdapterDictEntryModel.class, AdapterVersion.DictEntryTablePrefix,schemaVersion, "stdDictId="+orgCodeId+";adapterInfo<>0", null, null, null,false);
        list.forEach(adapterDictEntryModel ->{
            String entryCode = adapterDictEntryModel.getAdapterEntryCode();//平台org_code
            String stdEntryCode = adapterDictEntryModel.getStdEntryCode();//省平台org_code
            if(StringUtils.isNotEmpty(entryCode)){
                provinceOrgCache.put(entryCode,adapterDictEntryModel.getStdEntryCode());
            }
            if(StringUtils.isNotEmpty(stdEntryCode) && StringUtils.isNotEmpty(entryCode)){
                stdOrgCodeCache.put(stdEntryCode,entryCode);
            }
        });
        OrgAdapterCache.ORG_CACHE.put(schemaVersion,provinceOrgCache);
        OrgAdapterCache.STD_ORG_CACHE.put(schemaVersion,stdOrgCodeCache);
        logger.debug("InitAdapter  - 获取适配后的组织机构代码设置成缓存成功！");
    }
}