package com.yihu.hos.central.rest.services.crawler;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.ehr.elasticsearch.ElasticSearchUtil;
import com.yihu.hos.central.rest.common.format.AdapterBase;
import com.yihu.hos.central.rest.common.format.PreparData;
import com.yihu.hos.central.rest.common.format.StdBase;
import com.yihu.hos.central.rest.constants.RedisCollection;
import com.yihu.hos.central.rest.feign.SvrResourceServiceClient;
import com.yihu.hos.central.rest.models.crawler.adapter.AdapterDataSet;
import com.yihu.hos.central.rest.models.crawler.adapter.StdDataSet;
import com.yihu.hos.central.rest.models.standard.adapter.AdapterDatasetModel;
import com.yihu.hos.central.rest.models.standard.adapter.AdapterDictEntryModel;
import com.yihu.hos.central.rest.models.standard.adapter.AdapterSchemeModel;
import com.yihu.hos.central.rest.services.errorUpload.ErrorUploadService;
import com.yihu.hos.central.rest.services.qc.EsSimplePackage;
import com.yihu.hos.central.rest.services.standard.adapter.*;
import com.yihu.hos.central.rest.services.standard.standard.StdDatasetService;
import com.yihu.hos.central.rest.services.standard.standard.StdDictEntryService;
import com.yihu.hos.central.rest.services.standard.standard.StdMetadataService;
import com.yihu.hos.core.datatype.CollectionUtil;
import com.yihu.hos.core.datatype.DateUtil;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.model.Envelop;
import com.yihu.hos.web.framework.model.standard.AdapterVersion;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import com.yihu.hos.web.framework.util.springutil.SpringBeanUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

@Service
public class CrawlerManager {
    public static final String BEAN_ID = "qCCrawlerManager";
    public static final List<String> outpatient = Arrays.asList("BASEINFO", "OUTPATIENT", "OUTPATIENT_FEE", "OUTPATIENT_DIAG");
    public static final List<String> inpatient = Arrays.asList("BASEINFO", "INPATIENT", "INPATIENT_FEE", "INPATIENT_LONGORDER", "INPATIENT_PROORDER", "INPATIENT_INDIAG", "INPATIENT_FIRSTPAGE");
    public static final List<String> exam = Arrays.asList("BASEINFO", "HEALTHEXAM_REG", "HEALTHEXAM_CATALOG", "HEALTHEXAM_SUBITEM");
    public static final List<String> exclude = Arrays.asList("HDSD00_18", "HDSC00_04");//医疗资源不在hbase中
    private static final Map<String, String> DATASET_RECORDS = new ConcurrentHashMap<>();

    private static Logger logger = LoggerFactory.getLogger(CrawlerManager.class);
    @Resource
    private AdapterDatasetService adapterDatasetService;
    @Resource
    private AdapterMetadataService adapterMetadataService;
    @Resource
    private AdapterDictService adapterDictService;
    @Resource
    private AdapterSchemeVersionService adapterSchemeVersionService;
    @Resource
    private StdMetadataService stdMetadataService;
    @Resource(name = AdapterSchemeService.BEAN_ID)
    private AdapterSchemeService adapterSchemeService;
    @Resource(name = StdDatasetService.BEAN_ID)
    private StdDatasetService stdDatasetService;
    @Resource(name = StdDictEntryService.BEAN_ID)
    private StdDictEntryService stdDictEntryService;
    @Autowired
    private ElasticSearchUtil elasticSearchUtil;
    @Autowired
    protected ObjectMapper objectMapper;

    private static final String INDEX = "upload";
    private static final String TYPE_RECORD = "record";
    @Autowired
    private DataCollectDispatcher dataCollectDispatcher;
    @Autowired
    SvrResourceServiceClient resourceServiceClient;
    @Autowired
    private ErrorUploadService errorUploadService;
    @Value("${schemaVersion}")
    private String schemaVersion;
    @Value("${stdVersion}")
    private String stdVersion;
    @Value("${targetPlatform}")
    private String targetPlatform;


    //省平台错误数据重传
    public void failUpload() {

        if (!getDataForPrepare(null)) {
            logger.info("失败重传缓存默认适配版本失败");
            throw new RuntimeException("失败重传缓存默认适配版本失败");
        }
        RedisTemplate<String, Serializable> redisTemplate = SpringBeanUtil.getService("redisTemplate");
        Serializable serializable = redisTemplate.opsForList().rightPop(RedisCollection.PROVINCIAL_PLATFORM_QUEUE_FAIL);
        EsSimplePackage pack = null;
        try {
            if (serializable != null) {
                String packStr = serializable.toString();
                pack = objectMapper.readValue(packStr, EsSimplePackage.class);
                if (pack != null) {
                    // 上传数据到省平台
                    Boolean aBoolean = collectProcessByRowkey(pack);
                    if (!aBoolean) {
                        logger.error("上传失败数据重传省平台--失败~！");
                    }
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    //schemeVersion值为null时,初始化默认版本
    public Boolean getDataForPrepare(String schemeVersion) {
        if (StringUtils.isEmpty(schemeVersion)) {
            schemeVersion = schemaVersion;
        }
        if (PreparData.isPrepar(schemeVersion)) {
            return true;
        }
        AdapterBase.setStdDictEntryService(stdDictEntryService);
        AdapterBase.setStdDatasetService(stdDatasetService);
        AdapterBase.setStdMetadataService(stdMetadataService);
        AdapterBase.setAdapterMetadataService(adapterMetadataService);
        AdapterBase.setAdapterDictService(adapterDictService);
        AdapterBase.setAdapterSchemeVersionService(adapterSchemeVersionService);
        StdBase.setStdMetadataService(stdMetadataService);
        StdBase.setStdDatasetService(stdDatasetService);
        try {
            Map<String, AdapterDataSet> adapterDataSetMap = new HashMap<String, AdapterDataSet>();
            Map<String, StdDataSet> stdDataSetMap = new HashMap<String, StdDataSet>();
            StandardVersion targetVersion = null;
            StandardVersion originVersion = null;
            AdapterVersion adapterVersion = new AdapterVersion(schemeVersion);

            List<AdapterDatasetModel> adapterDataSetModelList = adapterDatasetService.getNeedCrawer(adapterVersion);
            if (adapterDataSetModelList != null && !adapterDataSetModelList.isEmpty()) {
                AdapterSchemeModel adapterScheme = adapterSchemeService.getEntity(AdapterSchemeModel.class, adapterDataSetModelList.get(0).getSchemeId());
                targetVersion = new StandardVersion(adapterScheme.getStdVersion());
                originVersion = new StandardVersion(adapterScheme.getAdapterStdVersion());
                PreparData.setTargetVersion(targetVersion);
                PreparData.setOriginVersion(originVersion);
            }
            /**
             * 字典项初始化
             */
            List<AdapterDictEntryModel> adapterDictEntryModelList = adapterDictService.getAdapterEntryDictNotNullList(adapterVersion);

            Map<Long, List<AdapterDictEntryModel>> adapterDictEntryModelMap = new HashMap<>();
            for (AdapterDictEntryModel adapterDictEntryModel : adapterDictEntryModelList) {
                List<AdapterDictEntryModel> entryModelList = adapterDictEntryModelMap.get(adapterDictEntryModel.getStdDictId());
                if (CollectionUtil.isEmpty(entryModelList)) {
                    entryModelList = new ArrayList<>();
                }
                entryModelList.add(adapterDictEntryModel);
                adapterDictEntryModelMap.put(adapterDictEntryModel.getStdDictId(), entryModelList);
            }


            /**
             * 数据集初始化
             */
            for (AdapterDatasetModel adapterDatasetModel : adapterDataSetModelList) {
                if (!StringUtils.isEmpty(adapterDatasetModel.getAdapterDatasetCode())) {//--->>为空的话是没有适配的,则无需缓存下来
                    //adapterDataSetMap   key值为:  省平台数据集编码@@@适配数据集编码@@@适配数据集主键id
                    //加上适配主键id,是因为个人基本信息-地址信息 多次适配了 人口学信息 (有不同的地址:工作地址,常住地址,产后地址等 需要进行上传)
                    adapterDataSetMap.put(adapterDatasetModel.getStdDatasetCode() + "@@@" + adapterDatasetModel.getAdapterDatasetCode() + "@@@" + adapterDatasetModel.getId(), new AdapterDataSet(adapterDatasetModel, adapterVersion, originVersion, targetVersion, adapterDictEntryModelMap));
                    //stdDataSetMap   key值为:省平台数据集编码@@@适配数据集主键id
                    stdDataSetMap.put(adapterDatasetModel.getStdDatasetCode() + "@@@" + adapterDatasetModel.getId(), new StdDataSet(adapterDatasetModel.getStdDatasetId(), targetVersion));
                }
            }
            Map<String, Map<String, AdapterDataSet>> adapterDataSetVersionMap = PreparData.getAdapterDataSetVersionMap();
            Map<String, Map<String, StdDataSet>> stdDataSetVersionMap = PreparData.getStdDataSetVersionMap();
            adapterDataSetVersionMap.put(schemeVersion, adapterDataSetMap);
            stdDataSetVersionMap.put(schemeVersion, stdDataSetMap);
            List<String> stdDataSets = new ArrayList<>();
            if (!CollectionUtil.isEmpty(adapterDataSetMap.keySet())) {
                //获取资源数据
                adapterDataSetMap.keySet().stream().forEach(key -> {
                    stdDataSets.add(key.split("@@@")[1].toUpperCase());
                });
            }
            exclude.forEach(key -> {
                stdDataSets.remove(key);
            });
            PreparData.getStdDataSetMap().put(schemeVersion, stdDataSets);
            PreparData.getAdapterFlg().put(schemeVersion, true);
            PreparData.getStdFlg().put(schemeVersion, true);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    //单个病人质控数据采集上传 - 省平台队列方式
    public Boolean collectProcessByRowkey(EsSimplePackage pack) {
        if (!PreparData.isPrepar(schemaVersion)) {
            throw new RuntimeException("适配方案未准备");
        }
        Map<String, Object> dataList = null;
        logger.trace("查询数据-->上传至中间库开始=======================================,");
        try {

            Map<String, AdapterDataSet> adapterDataSetMap = PreparData.getAdapterDataSetVersionMap().get(schemaVersion);
            Map<String, StdDataSet> stdDataSetMap = PreparData.getStdDataSetVersionMap().get(schemaVersion);
            try {
                List<String> stdDataSets = PreparData.getStdDataSetMap().get(schemaVersion);
                if (!CollectionUtil.isEmpty(stdDataSets)) {
                    //获取资源数据
                    long resourceStart = DateUtil.getSystemTime();
                    Envelop resource = resourceServiceClient.getResourceByDataSets(stdDataSets, "*", "*", "*", pack.getRowkey(), stdVersion);
                    long resourceEnd = DateUtil.getSystemTime();
                    logger.debug("获取资源所用时间："+(resourceEnd-resourceStart)/1000);

                    //2.  上传数据集
                    if (resource.getObj() != null) {
                        dataList = (Map<String, Object>) resource.getObj();
                    } else {
                        // 无数据,获取的数据有误，失败数据保存到es
                        pack.setFail_type(1);
                        errorUploadService.addUploadFailData(pack);
                        return false;
                    }

                    Map<String, Object> record = new HashMap<>();
                    String jkzlDataSetCode = "";
                    List<Map<String, Object>> data = null;
                    List<String> missDataSets = new ArrayList<>();
                    List<String> succDataSetList = new ArrayList<>();
                    List<Map<String, Object>> succDataSets = new ArrayList<>();
                    Map<String, Object> succDataSet = null;
                    //上传档案
                    for (String key : adapterDataSetMap.keySet()) {
                        jkzlDataSetCode = key.split("@@@")[1].toUpperCase();
                        String targetDataSet = key.split("@@@")[0].toUpperCase();
                        Object obj = dataList.get(jkzlDataSetCode);
                        if (obj == null) {
                            missDataSets.add(key.split("@@@")[0].toUpperCase());
                            //TODO 没数据，保存缺失数据集数据到es
                        } else {
                            data = (List<Map<String, Object>>) obj;
                            if (data.size() > 0) {
                                succDataSet = new HashMap<>();
                                int succ = dataCollectDispatcher.uploadDataByRowkeyAndImport(pack, data, adapterDataSetMap, stdDataSetMap, key);
                                if (succ == 0) {
                                    missDataSets.add(key.split("@@@")[0].toUpperCase());
                                } else {
                                    succDataSet.put(targetDataSet, succ);
                                    succDataSets.add(succDataSet);
                                    succDataSetList.add(targetDataSet);
                                    //添加数据集统计数据
                                    this.updateDatasetDetails(pack.getOrg_code(), DateUtil.getCurrentString(DateUtil.DEFAULT_YMDHMSDATE_FORMAT), PreparData.getTargetVersion().getVersion(),
                                            targetDataSet, pack.getEvent_type(), succ,targetPlatform);

                                }

                            } else {
                                missDataSets.add(key.split("@@@")[0].toUpperCase());
                            }

                        }
                    }
                    // 添加数据集，记录 record
                    record.put("_id", UUID.randomUUID().toString());
                    record.put("pack_id", pack.get_id());
                    record.put("pack_pwd", pack.getPwd());
                    record.put("missing", missDataSets);
                    record.put("dataset_count", succDataSets.size());
                    record.put("datasets", objectMapper.writeValueAsString(succDataSets));
                    record.put("rowkey", pack.getRowkey());
                    record.put("from_version", stdVersion);
                    record.put("version", PreparData.getTargetVersion().getVersion());
                    record.put("to_platform", targetPlatform);
                    record.put("org_code", pack.getOrg_code());
                    record.put("patient_id", pack.getPatient_id());
                    record.put("analyze_date", DateUtil.getCurrentString(DateUtil.DEFAULT_YMDHMSDATE_FORMAT));
                    record.put("event_no", pack.getEvent_no());
                    record.put("event_date", pack.getEvent_date());
                    record.put("event_type", pack.getEvent_type());
                    // 添加其他字段
                    record.put("patient_name", pack.getPatient_name());
                    record.put("idcard_no", pack.getIdcard_no());
                    record.put("org_name", pack.getOrg_name());
                    record.put("org_area", pack.getOrg_area());
                    int uploadStatus = checkUploadResult(succDataSetList, pack.getEvent_type());
                    record.put("upload_status", uploadStatus);
                    elasticSearchUtil.index(INDEX, TYPE_RECORD, record);
                    //上传成功删除 ES错误数据,失败添加错误数据
                    if (uploadStatus == 1 ){
                        errorUploadService.delUploadFailData(pack);
                    }else {
                        pack.setFail_type(3);
                        errorUploadService.addUploadFailData(pack);
                    }
                    logger.debug("上传省平台成功数据集数：" + succDataSets.size() + ",上传完整性,1为完整：：" + uploadStatus + " patient_id:" + pack.getPatient_id() + " ,event_no :" + pack.getEvent_no());
                }
            } catch (Exception e) {
                logger.error("档案数据上传失败",e);
                //添加失败信息到ES
                pack.setFail_type(2);
                errorUploadService.addUploadFailData(pack);
                return false;
            }
        } catch (Exception e) {
            logger.error("上传档案数据失败2", e);
            //添加失败信息到ES
            pack.setFail_type(2);
            errorUploadService.addUploadFailData(pack);
            return false;
        }
        return true;
    }


    /**
     * 通过上传成功的数据集，判断上传关联的数据集是否完整，如果不完成则表示档案上传失败
     *
     * @param succDataSets 上传成功的数据集
     * @param eventType    事件类型
     * @return 1 数据集完整 0 门诊数据集缺失，-1 住院数据集缺失，-2 体检数据集缺失，-3 事件类型有误
     */
    public int checkUploadResult(List<String> succDataSets, Integer eventType) {
        Integer flag = 1;
        //业务块必须的数据集校验
        //event_type = 0 是门诊，1 是住院，2 是体检
        if (eventType == 0) {
            if (!succDataSets.containsAll(outpatient)) {
                flag = 0;
            }
        } else if (eventType == 1) {
            if (!succDataSets.containsAll(inpatient)) {
                flag = -1;
            }
        } else if (eventType == 2) {
            if (!succDataSets.containsAll(exam)) {
                flag = -2;
            }
        } else {
            //事件类型不存在
            flag = -3;
        }
        return flag;
    }


    //获取数据集合的时候加锁
    public static synchronized Map<String, String> getDatasetDetails (){
        Map<String, String> temp = new HashMap<>();
        temp.putAll(DATASET_RECORDS);
        DATASET_RECORDS.clear();
        return temp;
    }

    //更新统计数据集详情
    private void updateDatasetDetails (String orgCode, String receiveDate, String version,
                                       String dataset, int eventType, int row,String toPlatform) {
        String date = receiveDate.substring(0, 10);
        StringBuilder record = new StringBuilder();
        record.append(orgCode)
                .append(";")
                .append(date)
                .append(";")
                .append(version)
                .append(";")
                .append(eventType)
                .append(";")
                .append(dataset)
                .append(":")
                .append(toPlatform);
        String val = DATASET_RECORDS.get(record.toString());
        if (val != null) {
            String [] _val = val.split(";");
            int count = Integer.parseInt(_val[0]) + 1;
            int rows = Integer.parseInt(_val[1]) + row;
            //双重检查，防止处理数据的时候保存数据的任务线程已经刷新了【DATASET_RECORDS】数据集合，降低前后数据不一致的概率
            if (DATASET_RECORDS.get(record.toString()) != null) {
                DATASET_RECORDS.put(record.toString(), count + ";" + rows);
            } else {
                DATASET_RECORDS.put(record.toString(), 1 + ";" + row);
            }
        } else {
            DATASET_RECORDS.put(record.toString(), 1 + ";" + row);
        }
    }




}
