package com.yihu.hos.central.rest.services.crawler;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.yihu.hos.central.rest.common.format.DataSetTransformer;
import com.yihu.hos.central.rest.common.util.ImportDataToDataBase;
import com.yihu.hos.central.rest.models.crawler.adapter.AdapterDataSet;
import com.yihu.hos.central.rest.models.crawler.adapter.StdDataSet;
import com.yihu.hos.central.rest.services.qc.EsSimplePackage;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.text.ParseException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;


@Service
public class DataCollectDispatcher {

    private static Logger logger = LoggerFactory.getLogger(DataCollectDispatcher.class);
    @Autowired
    private ObjectMapper objectMapper;

    private static final String TYPE_DATASET_COUNT = "dataset_count";
    private static final String INDEX = "upload";
    private static final String TYPE = "info";
    private static final String TYPE_RECORD = "record";


    /* ******************************　2018-06-12　队列方式上传省平台　　**************************************　　*/

    /**
     * 调用ehr查询主表和细表数据,肯定会返回  rowkey   org_code    event_no    event_date  event_type  这5个字段
     * @param pack                  省平台上传队列信息
     * @param detailModelList     资源数据
     * @param dataSetMap            数据集标准信息
     * @param stdDataSetMap         平台标准信息
     * @param uploadCode            上传的表
     * @return
     * @throws IOException
     * @throws ParseException
     */
    public int uploadDataByRowkeyAndImport(EsSimplePackage pack, List<Map<String,Object>>  detailModelList , Map<String, AdapterDataSet> dataSetMap, Map<String, StdDataSet> stdDataSetMap, String uploadCode) throws IOException, ParseException {
        try {
//            String version = SysConfig.getInstance().getVersionMap().get(SysConfig.orgcode);
            ArrayNode resultList = objectMapper.readValue(objectMapper.writeValueAsString(detailModelList), ArrayNode.class);
            int succCount = uploadDatasByDataset(pack,resultList, dataSetMap, stdDataSetMap, uploadCode);
            return succCount;

        } catch (Exception e) {
            e.printStackTrace();
        }
        return 0;
    }


    //上传档案数据到省平台
    public int uploadDatasByDataset(EsSimplePackage pack,ArrayNode dataMap, Map<String, AdapterDataSet> dataSetMap, Map<String, StdDataSet> stdDataSetMap, String uploadCode) {
        int result = 0;
        int preSize = dataMap.size();
        ArrayNode tempNode = dataMap;
        try {
            String provinceDatasetCode = "";
            //uploadCode 不做格式性校验,直接截取 ,节省效率
            try {
                provinceDatasetCode = uploadCode.split("@@@")[0].toUpperCase();
            } catch (Exception e) { //如果此处报错,极大可能是 格式有问题  uploadCode格式: 省平台数据集code@@@平台数据集code
                e.printStackTrace();
                ImportDataToDataBase.recordFailedInsert(pack,pack.getRowkey(),"上传省平台的数据集uploadCode格式有误",e,null);
                throw new RuntimeException("上传省平台的数据集uploadCode格式有误,uploadCode="+uploadCode);
            }
            String stdDataSetCode = dataSetMap.get(uploadCode).getAdapterDataSetT().getStdDatasetCode();

            //临时存储过滤后的需要上传的数据
            ArrayNode temp = objectMapper.createArrayNode();

            //如果待转换的数据为空了,直接返回了,没必要一下操作,浪费性能
            if (dataMap == null || dataMap.size() == 0) {
                return 0;
            }

            //TODO  临时处理-江西省平台诊断业务拆分， 暂时写死判断是出院/入院诊断
            if ("INPATIENT_OUTDIAG".equals(stdDataSetCode)) {//出院诊断
                for (JsonNode objectNode : dataMap) {
                    JsonNode typeNode = objectNode.get("HDSD00_69_008");
                    if (typeNode != null && StringUtils.isNotBlank(typeNode.asText())) {
                        if ("1".equals(typeNode.asText())) {
                            temp.add(objectNode);
                        }
                    } else {
                        temp.add(objectNode);
                    }
                }
                dataMap = objectMapper.createArrayNode();
                dataMap.addAll(temp);
            } else if ("INPATIENT_INDIAG".equals(stdDataSetCode)) { //入院诊断
                for (JsonNode objectNode : dataMap) {
                    JsonNode typeNode = objectNode.get("HDSD00_69_008");
                    if (typeNode != null && StringUtils.isNotBlank(typeNode.asText())) {
                        if ("3".equals(typeNode.asText())) {
                            temp.add(objectNode);
                        }
                    } else {
                        temp.add(objectNode);
                    }
                }
                dataMap = objectMapper.createArrayNode();
                dataMap.addAll(temp);
            }

            //对住院和门诊的进行判断筛选
            //event_type = 0 是门诊
            //event_type = 1 是住院
            //event_type = 2 是体检

            //-->住院
            if (provinceDatasetCode.contains("INPATIENT")) {
                for (JsonNode objectNode : dataMap) {
                    JsonNode jsonNode = objectNode.get("event_type");
                    if (jsonNode == null || StringUtils.isBlank(jsonNode.asText()) || "1".equals(jsonNode.asText())) {
                        temp.add(objectNode);
                    }
                }
                dataMap = objectMapper.createArrayNode();
                dataMap.addAll(temp);

                //--->门诊
            } else if (provinceDatasetCode.contains("OUTPATIENT") || "EDS_EM_OBSERVATION_MED_RECORD".equals(provinceDatasetCode) || "EDS_OUTPAT_MEDICAL_RECORD".equals(provinceDatasetCode) || "EDS_DISP_PRES_MASTER".equals(provinceDatasetCode)) {
                for (JsonNode objectNode : dataMap) {
                    JsonNode jsonNode = objectNode.get("event_type");
                    if (jsonNode == null || StringUtils.isBlank(jsonNode.asText()) || "0".equals(jsonNode.asText())) {
                        temp.add(objectNode);
                    }
                }
                dataMap = objectMapper.createArrayNode();
                dataMap.addAll(temp);

                //--->>体检
            } else if (provinceDatasetCode.contains("HEALTHEXAM")) {
                for (JsonNode objectNode : dataMap) {
                    JsonNode jsonNode = objectNode.get("event_type");
                    if (jsonNode == null || StringUtils.isBlank(jsonNode.asText()) || "2".equals(jsonNode.asText())) {
                        temp.add(objectNode);
                    }
                }
                dataMap = objectMapper.createArrayNode();
                dataMap.addAll(temp);
            }

            if (dataMap.size() > 0) {
                // 数据标准转换，写入数据到省平台
                DataSetTransformer dataTransformer = new DataSetTransformer();
                dataTransformer.setBeforeTranData(dataMap);
                dataTransformer.transfer(pack,dataSetMap, uploadCode);//TODO 效率改进
//                AdapterDataSet adapterDataSet = dataSetMap.get(originDataSetCode);
//                dataTransformer.transferTimeFormat(stdDataSetMap, adapterDataSet.getAdapterDataSetT().getStdDatasetCode());
                dataTransformer.setDataSetCode(stdDataSetCode);
                result = uploadByDataSet(pack,dataTransformer); //上传数据到省平台，返回上传成功数

            }else {
                logger.debug(uploadCode+"：转换前size："+ preSize + ",================ 错误：dataMap 转换后为空!"+tempNode.toString());
            }
        } catch (Exception e) {
            ImportDataToDataBase.recordFailedInsert(pack,pack.getRowkey(),"uploadDatasByDataset上传省平台异常",e,null);
            e.printStackTrace();
            result = 0;
        }
        return result;
    }


    /**
     * 上报数据
     *
     * @return
     */
    public int uploadByDataSet(EsSimplePackage pack,DataSetTransformer dataTransformer) throws Exception {
        int successCount = 0;//成功数
        CopyOnWriteArrayList<JsonNode> arrayNode = dataTransformer.getAfterTranData();
        if (arrayNode != null && arrayNode.size() > 0) {
             //使用单条插入方式执行sql语法
            for (JsonNode objectNode : arrayNode) {
                try {
                    //TODO  需要添加rowkey
                    String errorMsg = ImportDataToDataBase.exportSql(pack,objectNode.get("rowkey").asText(),dataTransformer.getDataSetCode(), "PK_" + dataTransformer.getDataSetCode(), objectNode);
                    if (errorMsg == null) {
                        // 上传省平台成功，添加成功数
                        successCount++;
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
                 //上传完成后,将转换后的数据设置为空
        }

        return successCount;
    }

}