package com.yihu.hos.central.rest.services.upload;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.ehr.elasticsearch.ElasticSearchUtil;
import com.yihu.hos.central.rest.cache.OrgAdapterCache;
import com.yihu.hos.central.rest.feign.RedisClient;
import com.yihu.hos.central.rest.models.crawler.statistics.UploadDetail;
import com.yihu.hos.central.rest.services.crawler.CrawlerManager;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Page;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;


@Service
public class UploadDetailService {
    private static Logger logger = LoggerFactory.getLogger(UploadDetailService.class);
    @Autowired
    private JdbcTemplate tempJdbcTemplate;

    @Autowired
    private ElasticSearchUtil elasticSearchUtil;

    @Autowired
    private RedisClient redisClient;

    @Value("${schemaVersion}")
    private String schemaVersion;

    private SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");

    @Autowired
    private ObjectMapper objectMapper;

    private static final String INDEX = "upload_detail";
    private static final String TYPE = "info";

    public void saveOrUpdate(Date recordDate) throws Exception {
        //根据recordDate删除记录
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String format = sdf.format(recordDate);
        elasticSearchUtil.deleteByField(INDEX, TYPE, "record_date", format);

        //获取各个机构 省平台门诊量,住院量,体检量,以及已上传的日期
        List<Map<String, Object>> datas = tempJdbcTemplate.queryForList("SELECT t1.org_code,t2.inp,t2.inpDate,t3.outp,t3.outpDate,t4.heal,t4.healDate FROM (SELECT org_code FROM inpatient UNION SELECT org_code FROM outpatient UNION SELECT org_code FROM HEALTHEXAM_REG) t1 LEFT JOIN   (SELECT org_code, COUNT(*) AS inp,TO_CHAR(max(Discharge_Date),'YYYY-MM-DD') AS inpDate  FROM inpatient GROUP BY org_code) t2 ON t2.org_code = t1.org_code LEFT JOIN (SELECT org_code, COUNT(*) AS outp ,TO_CHAR(max(Visit_Dtime),'YYYY-MM-DD') AS outpDate FROM outpatient GROUP BY org_code) t3  ON t3.org_code = t1.org_code LEFT JOIN (SELECT org_code, COUNT(*) AS heal ,TO_CHAR(max(Start_Dtime),'YYYY-MM-DD') AS healDate FROM HEALTHEXAM_REG GROUP BY org_code) t4  ON t4.org_code = t1.org_code");

        Map<String, Object> all = tempJdbcTemplate.queryForMap("SELECT sum(t2.inp) as toInp,max(t2.inpDate) as toInpData,sum(t3.outp) as toOutp ,max(t3.outpDate) as toOutpDate ,sum(t4.heal) as toHeal,max(t4.healDate) as toHealDate FROM (SELECT org_code FROM inpatient UNION SELECT org_code FROM outpatient UNION SELECT org_code FROM HEALTHEXAM_REG) t1 LEFT JOIN   (SELECT org_code, COUNT(*) AS inp,TO_CHAR(max(Discharge_Date),'YYYY-MM-DD') AS inpDate  FROM inpatient GROUP BY org_code) t2 ON t2.org_code = t1.org_code LEFT JOIN (SELECT org_code, COUNT(*) AS outp ,TO_CHAR(max(Visit_Dtime),'YYYY-MM-DD') AS outpDate FROM outpatient GROUP BY org_code) t3  ON t3.org_code = t1.org_code LEFT JOIN (SELECT org_code, COUNT(*) AS heal ,TO_CHAR(max(Start_Dtime),'YYYY-MM-DD') AS healDate FROM HEALTHEXAM_REG GROUP BY org_code) t4  ON t4.org_code = t1.org_code");
        //保存
        if (datas != null) {
            for (Map<String, Object> data : datas) {
                String orgCode = data.get("org_code") + "";
                //通过省平台的org_code转换成 平台标准的org_code ,如果转换失败,则保留原值
                if (!StringUtils.isEmpty(orgCode)) {
                    String stdOrgCode = OrgAdapterCache.getStdOrgCode(schemaVersion, orgCode);
                    if (!StringUtils.isEmpty(stdOrgCode)) {
                        orgCode = stdOrgCode;
                    }
                }

                Long inpNum = stringToLong(data.get("inp") + "");
                Date inpDate = stringToDate(data.get("inpDate") + "");
                Long outpNum = stringToLong(data.get("outp") + "");
                Date outpDate = stringToDate(data.get("outpDate") + "");
                Long healNum = stringToLong(data.get("heal") + "");
                Date healDate = stringToDate(data.get("toHealDate") + "");

                //保存记录
                UploadDetail uploadDetail = new UploadDetail();
                uploadDetail.setRecord_date(recordDate);
                uploadDetail.setInpatient_num(inpNum);
                uploadDetail.setInpatient_date(inpDate);
                uploadDetail.setOutpatient_num(outpNum);
                uploadDetail.setOutpatient_date(outpDate);
                uploadDetail.setHealth_num(healNum);
                uploadDetail.setHealth_date(healDate);
                uploadDetail.setOrg_code(orgCode);
                uploadDetail.setUpdate_time(new Date());
                elasticSearchUtil.index(INDEX, TYPE, objectMapper.readValue(objectMapper.writeValueAsString(uploadDetail), Map.class));
            }
        }
        if (all != null) {
            Long inpNum = stringToLong(all.get("toInp") + "");
            Date inpDate = stringToDate(all.get("toInpData") + "");
            Long outpNum = stringToLong(all.get("toOutp") + "");
            Date outpDate = stringToDate(all.get("toOutpDate") + "");
            Long healNum = stringToLong(all.get("toHeal") + "");
            Date healDate = stringToDate(all.get("healDate") + "");

            //保存记录
            UploadDetail uploadDetail = new UploadDetail();
            uploadDetail.setRecord_date(recordDate);
            uploadDetail.setInpatient_num(inpNum);
            uploadDetail.setInpatient_date(inpDate);
            uploadDetail.setOutpatient_num(outpNum);
            uploadDetail.setOutpatient_date(outpDate);
            uploadDetail.setHealth_num(healNum);
            uploadDetail.setHealth_date(healDate);
            uploadDetail.setOrg_code("ALL");
            uploadDetail.setUpdate_time(new Date());
            elasticSearchUtil.index(INDEX, TYPE, objectMapper.readValue(objectMapper.writeValueAsString(uploadDetail), Map.class));
        }
    }

    private Long stringToLong(String str) {
        Long num = 0L;
        if (!StringUtils.isEmpty(str) && !"null".equalsIgnoreCase(str)) {
            num = Long.valueOf(str);
        }
        return num;
    }

    //str 格式为 yyyy-dd-mm
    private Date stringToDate(String str) throws ParseException {
        Date d = null;
        if (!StringUtils.isEmpty(str) && !"null".equalsIgnoreCase(str)) {
            d = simpleDateFormat.parse(str);
        }
        if (d == null) {
            d = new SimpleDateFormat("yyyy-MM-dd").parse("2017-01-01");
        }
        return d;
    }


    /**
     * 生成省平台上传-  数据集统计数据
     * 抽取date时间内的统计数据，存储到es中
     *
     * @param date 抽取时间
     * @return
     * @throws Exception
     */
    public com.yihu.ehr.util.rest.Envelop uploadDatasetDetail(String date) throws Exception {
        com.yihu.ehr.util.rest.Envelop envelop = new com.yihu.ehr.util.rest.Envelop();
        List<String> field = new ArrayList<>();
        field.add("org_code");
        String sqlOrg = "SELECT org_code FROM upload/record where analyze_date>= '" + date + " 00:00:00' AND analyze_date<='" + date + " 23:59:59' group by org_code";
        List<Map<String, Object>> orgList = elasticSearchUtil.findBySql(field, sqlOrg);
        for (Map<String, Object> orgMap : orgList) {
            String orgCode = orgMap.get("org_code") + "";
            List<Map<String, Object>> res = new ArrayList<>();
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.append("analyze_date>=" + date + " 00:00:00;");
            stringBuilder.append("analyze_date<" + date + " 23:59:59;");
            stringBuilder.append("org_code=" + orgCode);
            long starttime = System.currentTimeMillis();
            int count = (int) elasticSearchUtil.count("upload", "record", stringBuilder.toString());
            double pageNum = count % 1000 > 0 ? count / 1000 + 1 : count / 1000;
            for (int i = 0; i < pageNum; i++) {
                Page<Map<String, Object>> list = elasticSearchUtil.page("upload", "record", stringBuilder.toString(), i + 1, 1000);
                logger.info("查询耗时：" + (System.currentTimeMillis() - starttime) + "ms");
                for (Map<String, Object> map : list) {
                    if (map.get("datasets") != null) {
                        List<Map<String, Object>> dataSets = objectMapper.readValue(map.get("datasets").toString(), List.class);
                        for (Map<String, Object> dataSet : dataSets) {
                            for (Map.Entry<String, Object> entry : dataSet.entrySet()) {
                                getProvincePlatformDataSets(map, entry.getKey(), (int) entry.getValue(), res, date, orgCode);
                            }
                        }
                    }
                }
            }

            elasticSearchUtil.bulkIndex("upload", "qc_dataset_detail", res);
            logger.info("统计耗时：" + (System.currentTimeMillis() - starttime) + "ms");
        }
        envelop.setSuccessFlg(true);
        return envelop;
    }


    /**
     * 获取省平台上传-  数据集统计数据
     *
     * @param dataSet   数据集编码
     * @param row       行数
     * @param res       返回结果
     * @param date
     * @param orgCode
     * @param eventType
     */
    public void getProvincePlatformDataSets(Map<String, Object> paramMap, String dataSet, int row, List<Map<String, Object>> res, String date, String orgCode) {
        boolean flag = true;
        String eventType = paramMap.get("event_type").toString();
        String version = paramMap.get("version") + ""; //省平台版本号
        String toPlatform = paramMap.get("to_platform").toString();
        for (Map<String, Object> map : res) {
            if (dataSet.equals(map.get("dataset")) && eventType.equals(map.get("event_type"))) {
                flag = false;
                map.put("row", (int) map.get("row") + row);
                map.put("count", (int) map.get("count") + 1);
                break;
            }
        }
        if (flag) {
            Map<String, Object> map = new HashMap<>();
            map.put("org_code", orgCode);
            map.put("event_type", eventType);
            map.put("analyze_date", date + " 00:00:00");
            map.put("dataset", dataSet);
            map.put("dataset_name", redisClient.get("std_data_set_" + version + ":" + dataSet + ":name"));
            map.put("row", row);
            map.put("count", 1);
            map.put("to_platform", toPlatform);
            res.add(map);
        }
    }


}